/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/* Applet to draw rods stacked one on top of the other */

package RodStacker;

import java.applet.Applet;
import java.awt.*;
import hamlet.*;
import RodStacker.*;

class point
{
  public int x, y, radius, indexl, indexr; //all pixels
  public boolean selected;
  public point left, right;
	
  public point(int ix, int iy, int iradius, point ileft, 
	       point iright, int iindexl, int iindexr)
  {
    x = ix;
    y = iy;
    radius = iradius;
    left = ileft;
    right = iright;
    indexl = iindexl;
    indexr = iindexr;
  }

  public void print()
  {
    System.out.println("Point: x= " + x + " y= " + y + " radius= " + radius);
  }
}

class RodPanel extends Panel
{
  private final static short NUM_POINTS = 8;
  private final static int FRAME_HEIGHT = 350;
  private final static int FRAME_WIDTH = 400;
  private final static int L_MARG = 10;
  private final static int R_MARG = 10;
  private final static int T_MARG = 60;
  private final static int B_MARG = 10;
  private final static point ip1 = 
                      new point(20+1, 20+1, 20, null, null, -1, -1);
  private final static point ip2 = 
                      new point(FRAME_WIDTH / 2, 40+1, 40, null, null, -1, -1);
  private final static point ip3 = 
                      new point(FRAME_WIDTH - 15 - 1, 15+1, 15, 
				null, null, -1, -1);

  private int cx, cy; // the original center of the circle
  private int lastx, lasty; //the last x and y positions recorded by mouseDrag
  private Dimension currentSize;
  private int selIndex, dragCount;
  private point coords[];
//  private point prevCoords[];
  private Graphics offGraphics;
  private Image offImage;
	
  public RodPanel()
  {
//  	prevCoords = new point[NUM_POINTS];
//  	for (int i = 0; i < NUM_POINTS; i++)
//set up dummy values for the initial painting
//  		prevCoords[i] = new point(0,0,0,null,null); 
    currentSize = new Dimension(FRAME_WIDTH, FRAME_HEIGHT);
    InitRods(currentSize);
  }
  
  public synchronized void reshape(int x, int y, int width, int height)
  {
 /*   double scale = width / ((double)currentSize.width);
    
    currentSize.width = width; currentSize.height = height;
    coords[1].x = width / 2;
    coords[2].x = width - 15 - 1;
    for (int i = 0; i < NUM_POINTS; i++)
      coords[i].radius = (int)Math.round(coords[i].radius * scale);
      ReCalcPoints();*/
    super.reshape(x,y,width,height);

    currentSize.width = width;
    currentSize.height = height;
    try{ //at startup the system balks at creating an image
      offImage = createImage(width, height);
      offGraphics = offImage.getGraphics();
      offGraphics.setFont(getGraphics().getFont());
    }
    catch (Exception e){
      offImage = null;
      offGraphics = null;
    }
    reset();
  }
  
  void InitRods(Dimension s)
  {
    double scale = s.width/ ((double) FRAME_WIDTH);
    coords = new point[NUM_POINTS];
    coords[0] = new point((int)Math.round(20*scale + L_MARG), 
			  (int)Math.round(20*scale + B_MARG), 
			  (int)Math.round(20*scale), null, null, -1, -1);
    coords[1] = new point(s.width / 2, 
			  (int)Math.round(40*scale + B_MARG), 
			  (int)Math.round(40*scale), null, null, -1, -1);
    coords[2] = new point((int)Math.round(s.width - scale*15 - R_MARG), 
			  (int)Math.round(15*scale + B_MARG), 
			  (int)Math.round(15*scale), null, null, -1, -1);
      //set up some dummy values
    for (int i = 3; i < NUM_POINTS; i++)
      coords[i] = new point(0,0,0,null,null, -1, -1); 
    coords[3].left = coords[0]; coords[3].indexl = 0;
    coords[3].right = coords[1]; coords[3].indexr = 1;
    coords[4].left = coords[1]; coords[4].indexl = 1;
    coords[4].right = coords[2]; coords[4].indexr = 2;
    coords[5].left = coords[3]; coords[5].indexl = 3;
    coords[5].right = coords[4]; coords[5].indexr = 4;
    coords[6].left = coords[5]; coords[6].indexl = 5;
    coords[6].right = coords[4]; coords[6].indexr = 4;
    coords[7].left = coords[3]; coords[7].indexl = 3;
    coords[7].right = coords[6]; coords[7].indexr = 6;
    selIndex = -1;
    CalcPoints((int)Math.round(scale*85), (int)Math.round(scale*80), 
	       (int)Math.round(scale* 33), (int)Math.round(scale*40), 
	       (int)Math.round(scale*35));
  }
  
public void reset()
  {
    currentSize.height = size().height;
    currentSize.width = size().width;
    InitRods(size());
    repaint();
  }

  public Dimension preferredSize()
  {
    return new Dimension(FRAME_WIDTH, FRAME_HEIGHT);
  }
  
  public Dimension minimumSize()
  {
  	return new Dimension(FRAME_WIDTH, FRAME_HEIGHT);
  }

  public void CalcPoints(int r4, int r5, int r6, int r7, int r8)
  {
    CalcXY(coords[3], r4);
    CalcXY(coords[4], r5);
    CalcXY(coords[5], r6);
    CalcXY(coords[6], r7);
    CalcXY(coords[7], r8);
  }
	
  public void ReCalcPoints()
  {
    CalcXY(coords[3], coords[3].radius);
    CalcXY(coords[4], coords[4].radius);
    CalcXY(coords[5], coords[5].radius);
    CalcXY(coords[6], coords[6].radius);
    CalcXY(coords[7], coords[7].radius);
  }
  
  /*
  CheckValidity makes sure that no rods are intersecting each other or
  overflowing the limits of the box.  If two rods touch, then this function
  automatically connects the two rods.
  
  Returns:
  		0 if the points are not valid
  		1 if the points are valid and the structure of the rods 
		  has not changed
  		2 if the points are valid and the structure of the rods 
		  has changed
  */
  public int CheckValidity(point inCoords[])
  {
    int i, j, returnint = 0;
    boolean valid = true;
    boolean structureChanged = false;
    
    for (i = 0; i < NUM_POINTS && valid; i++)
      {
		if ((inCoords[i].x + inCoords[i].radius > size().width) ||
	    	(inCoords[i].x - inCoords[i].radius < 0) ||
		   (inCoords[i].y + inCoords[i].radius > size().height - 70) ||
		    (inCoords[i].y - inCoords[i].radius < 0))
		{
		  valid = false;
		}
		else if (inCoords[i].radius <= 5)
		{
			valid = false;
		}
		for (j = i+1; j < NUM_POINTS && valid; j++)
		  {
//  	System.out.println(i + "  " + j + "  " + 
//		distance(inCoords[i].x, inCoords[i].y, 
//		inCoords[j].x, inCoords[j].y) + "  " + 
//		(inCoords[i].radius + inCoords[j].radius));
		    if (distance(inCoords[i].x, inCoords[i].y, 
				 inCoords[j].x, inCoords[j].y) 
		    	< (inCoords[i].radius + inCoords[j].radius))
		    {
		    	structureChanged = true;		    	
			  //inCoords[i] is on top of inCoords[j]
			if (inCoords[i].y > inCoords[j].y) 
			    //inCoords[j] is to the left
			  if (inCoords[j].x < inCoords[i].x) 
			    {
			      inCoords[i].left = inCoords[j];
			      inCoords[i].indexl = j;
			    }
			  else
			    {
			      inCoords[i].right = inCoords[j];
			      inCoords[i].indexr = j;
			    }
			else //inCoords[j] is on top of inCoords[i]
			    //inCoords[i] is to the left
			  if (inCoords[i].x < inCoords[j].x) 
			    {
			      inCoords[j].left = inCoords[i];
			      inCoords[j].indexl = i;
			    }
			  else
			    {
			      inCoords[j].right = inCoords[i];
			      inCoords[j].indexr = i;
			    }
	    	  //valid = false;
	    	  //break;
	    	}
//    	else if (distance(inCoords[i].x, inCoords[i].y, inCoords[j].x, 
//	inCoords[j].y) < (inCoords[i].radius + inCoords[j].radius))
//	    		valid = false;
		  }
      }
      
    if (!valid)
    	returnint =  0;
    else if (valid && !structureChanged)
    	returnint = 1;
    else if (valid && structureChanged)
    	returnint = 2;
    return returnint;
  }
  
  public boolean RCheckValidity(point p)
  {
  	if (p.left != null && p.right != null)
  	{
//	System.out.println(distance(p.x, p.y, p.left.x, p.left.y) + 
//	  "  " + (p.radius + p.left.radius));
	  return ((distance(p.x, p.y, p.left.x, p.left.y) == 
		                       p.radius + p.left.radius) && 
  		  (distance(p.x, p.y, p.right.x, p.right.y) == 
		                        p.radius + p.right.radius) &&
  		  RCheckValidity(p.left) && RCheckValidity(p.right));
  	}
  	else
	  	return true;
  }
  
  public void CalcXY(point p, int radius)
  {
    double a = radius + p.left.radius;
    double b = radius + p.right.radius;
    double c = Math.sqrt(((p.right.x - p.left.x) * (p.right.x - p.left.x)) + 
			 ((p.right.y - p.left.y) * (p.right.y - p.left.y)));
    double d = p.right.x - p.left.x;
    double e = p.right.y - p.left.y;
    double sinB = e / c;
    double cosB = d / c;
    double cosA = (a * a + c * c - b * b) / (2 * a * c);
    double sinA = 1 - (cosA * cosA);
    if (sinA < 0){
//      System.out.println("Major Failure!!! #2 cosA= "+ cosA + " " +
//			 a + "  " + b + "  " + c);
//      p.print();
//      p.left.print();
//      p.right.print();
      sinA = 0; //this is to combat some sort of math error that I
                //keep on getting.  Sometimes cosA > 1
    }
    else
      sinA = Math.sqrt(sinA);
	
	p.x = (int) Math.round(p.left.x + a * (cosA*cosB - sinA*sinB));
	p.y = (int) Math.round(p.left.y + (a *(sinA*cosB + sinB*cosA)));
	p.radius = radius;
  }
  
  public void update(Graphics g)
  {
  	int i;

  	paint(g);
  }
  
  public void paint(Graphics g)
  {
  	if (offImage == null)
  	{
  	  	offImage = createImage(size().width, size().height);
 	 	offGraphics = offImage.getGraphics();
 	 	offGraphics.setFont(g.getFont());
 	}
 	offGraphics.setColor(getBackground());
	  //clear out the previous circles
 	offGraphics.fillRect(0, 0, size().width, size().height);
 	offGraphics.setColor(Color.black);
    offGraphics.drawRect(0,0,size().width, size().height);
    for (int i = 0; i < NUM_POINTS; i++)
    {
    	if (i == selIndex)
	  {
	    offGraphics.setColor(Color.red);
	    offGraphics.fillOval(coords[i].x - 2, size().height - 
				 coords[i].y - 2, 4, 4);
	  }
	try{
	  if (coords[i] == coords[selIndex].left || 
	      coords[i] == coords[selIndex].right)
	    offGraphics.setColor(Color.magenta);
	}
	  //if nothing is selected, selIndex == -1
	catch (ArrayIndexOutOfBoundsException e) 
		{
		}
		offGraphics.drawOval(coords[i].x-coords[i].radius, 
			size().height - coords[i].y - coords[i].radius, 
			coords[i].radius*2, coords[i].radius*2);
		offGraphics.setColor(Color.black);
	}
	g.drawImage(offImage, 0, 0, getBackground(), null);
  }
  
  //returns the array index of the circle clicked in by the user
  public int PtInCircle(int x, int y)
  {
    boolean found = false;
    int index = 0;
    int tx = x, ty = size().height - y;
    
    try{
      while (found == false)
	{
	  found = (Math.sqrt((tx - coords[index].x) * (tx - coords[index].x) 
			     + (ty - coords[index].y) * 
			     (ty - coords[index].y)) < coords[index].radius);
	  index++;
	}
    }
    catch (ArrayIndexOutOfBoundsException e){
      index = 0;
    }
      //we don't want to be able to select the bottom 3 rods
    if (index-1 >= 0 && index-1 <= 2) 
      return -1;
    else
      return index - 1;
  }
		
  public boolean mouseDown(Event evt, int x, int y)
  {
    //unselect all circles
    dragCount = 0;
    for (int i = 0; i < NUM_POINTS; i++)
      coords[i].selected = false;
    if (PtInCircle(x, y) != -1)
    {
		selIndex = PtInCircle(x, y);
		cx = lastx = coords[selIndex].x;
		cy = lasty = coords[selIndex].y;
    }
    else
    	selIndex = -1;
    repaint();
    return false;
  }
  
/*  public boolean mouseUp(Event evt, int x, int y)
  {
    selIndex = -1;
    return false;
  }
*/  
  public boolean mouseDrag(Event evt, int x, int y)
  {
    int i, result = 2, tc = 0;
    int tx = x; int ty = size().height - y;
    int delta, saveRad;
    boolean structureChanged = false;
    point tempA[] = new point[NUM_POINTS];
    
    if (++dragCount > 1)
    {
      if (selIndex != -1)
    	{
	  for (i = 0; i < 3; i++)
	    tempA[i] = coords[i]; //copy the fundamentals
	  for (i = 3; i < NUM_POINTS; i++)
	    {
	      tempA[i] = new point(coords[i].x, coords[i].y, 
				   coords[i].radius, null, null, 
				   coords[i].indexl, coords[i].indexr);
	    }
	  for (i = 3; i < NUM_POINTS; i++)
	    {
	      tempA[i].left = tempA[tempA[i].indexl];
	      tempA[i].right = tempA[tempA[i].indexr];
	    }
	  saveRad = coords[selIndex].radius;
	  coords[selIndex].radius += (ty - lasty);
	  /*			if (ty < lasty)
				coords[selIndex].radius -= 1;
			else
				coords[selIndex].radius += 1;
				*/
	  while (result != 0 && result != 1 && ++tc < 10)
	    {
	      ReCalcPoints();
	      result = CheckValidity(coords);
	    }
	  if (result == 1)
	    repaint();
	  else
	    {
	      for (i = 3; i < NUM_POINTS; i++)
		{
		  coords[i].left = null;
		  coords[i].right = null;
		}
	      for (i = 3; i < NUM_POINTS; i++)
		{
		  coords[i] = tempA[i];
		}
	      tempA = null;
	    }
	  lastx = tx;
	  lasty = ty;
	}
    }
    return false; 
  }
  
  //finds the number of pixels distance between (x1, y1) and (x2, y2)
  public int distance(int x1, int y1, int x2, int y2)
  {
    return (int) 
          Math.round(Math.sqrt((x2 - x1)*(x2 - x1) + (y2 - y1)*(y2 - y1)));
  }
  
  //returns a copy of a point
/*  public point CopyPoint(point source)
  {
  	if (source.left != null && source.right != null)
  	{
    	return new point(source.x, source.y, source.radius, 
	                 CopyPoint(source.left), CopyPoint(source.right));
    }
    else
    	return new point(source.x, source.y, source.radius, null, null);
  }
*/  
  public point ReturnSelected()
  {
  	if (selIndex > -1)
  		return coords[selIndex];
  	else
  		return null;
  }
}
