/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
<applet 
    CODEBASE=".."
    CODE="hamlet.AppButton.class" NAME="RodStacker.RodStacker"
    width=165 height=25 align=absbottom>
    <param name=LABEL value="Rod Stacking Simulation">
    A Java applet should appear here
</applet>
*/

package RodStacker;

import java.applet.Applet;
import java.awt.*;
import hamlet.*;
import RodStacker.*;

public class RodStacker extends AppFrame
{
private RodPanel myPanel;
private TrigPanel tPanel;
private AnswerFrame answers;
private MenuItem ansMenuItem;
  
public RodStacker()
  {
    Menu windowMenu = new Menu("Window");

    setTitle("The Rod Stacker");
    
    setLayout(new GridLayout(1,2));
    myPanel = new RodPanel();
    add(myPanel);
    
    tPanel = new TrigPanel();
    add(tPanel);
    
    answers = new AnswerFrame("Answers", 0, 0, 0, 0, 0, 0, 0, 0, 0, this);
    
    ansMenuItem = new MenuItem("Show Answers");
    windowMenu.add(ansMenuItem);
    menuBar.add(windowMenu);
    
    helpText = 
      "This applet dynamically simulates the Rod Stacking problem"	+"\n"+
      "found in Chapter 13 of \"Introduction to Scientific"		+"\n"+
      "Programming\" by Joseph L. Zachary.\n"				+"\n"+ 
      //								//
      "File Menu\n"							+"\n"+
      "   + Reset:  Returns to the default rod configuration"		+"\n"+
      "   + Refresh: Refreshes the applet"				+"\n"+
      "   + Hide:  Removes the applet from the display" 		+"\n"+
      "   + Quit:  Terminates the applet\n"				+"\n"+
      //								//
      "Window Menu"							+"\n"+
      "   + Show Answers: Shows the (x,y) coordinates of the top"	+"\n"+
      "      rod and all the intermediate variables used to calculate"	+"\n"+
      "      the position of the top rod (as found in the mathematical"	+"\n"+
      "      derivation in Chapter 13)."				+"\n"+
      "   + Hide Answers: Hides the answer window.  This option only"	+"\n"+
      "      comes up when you have chosen to show the Answer Window\n"	+"\n"+
      //								//
      "Display"								+"\n"+
      "   + Left Side: Clicking on a circle in the left panel will"	+"\n"+
      "      cause the \"rod\" to be selected.  When a rod is selected,"+"\n"+
      "      a detailed representation of the selected rod and its"	+"\n"+
      "      support rods appears in the right panel of the applet"	+"\n"+
      "      window.    Clicking and dragging the mouse inside one of"	+"\n"+
      "      the rods in the left panel will change the radius of that"	+"\n"+
      "      rod.  This may also change the position of rods which"	+"\n"+
      "      are supported by the chosen rod."				+"\n"+
      "   + Right Side: This detailed picture shows the (x,y)"		+"\n"+
      "      coordinates of the support rods and their radii as well"	+"\n"+
      "      as the radius of the chosen rod."				+"\n"+
      "";

    pack();
    validate();
  }
  
public Dimension preferredSize()
  {
    return new Dimension(640,450);
  }
  
public Dimension minimumSize()
  {
    return new Dimension(640, 450);
  }
  
public void reset()
  {
    myPanel.reset();
    tPanel.reset();
  }


public void HandlePointUpdate(point p)
  {
    tPanel.SetPoints(p, p.left, p.right);
    tPanel.repaint();
    CalcAndSetAnswers(p);
  }
	
public boolean mouseDown(Event evt, int x, int y)
  {
    point temp = myPanel.ReturnSelected();
    if (temp != null)
      {
	HandlePointUpdate(temp);
      }
    else
      {
	tPanel.SetPoints(null, null, null);
	tPanel.repaint();
	answers.SetAnswers(0, 0, 0, 0, 0, 0, 0, 0, 0);
      }
    return true;
  }
  
public void CalcAndSetAnswers(point p)
  {
    double a = p.radius + p.left.radius;
    double b = p.radius + p.right.radius;
    double c = Math.sqrt(((p.right.x - p.left.x) * (p.right.x - p.left.x)) + 
			 ((p.right.y - p.left.y) * (p.right.y - p.left.y)));
    double d = p.right.x - p.left.x;
    double e = p.right.y - p.left.y;
    double sinB = e / c;
    double cosB = d / c;
    double cosA = (a * a + c * c - b * b) / (2 * a * c);
    double sinA = Math.sqrt(1 - (cosA * cosA));
		
    answers.SetAnswers(a, b, c, d, e, cosA, cosB, sinA, sinB);
  }
  
public boolean mouseUp(Event evt, int x, int y)
  {
    point temp = myPanel.ReturnSelected();
    
    if (temp != null)
      {
	HandlePointUpdate(temp);
      }
    else
      {
	tPanel.SetPoints(null, null, null);
	tPanel.repaint();
      }
    return true;
  }
	
public boolean action(Event evt, Object what)
  {
    boolean handled = false;
    
    if (evt.target instanceof MenuItem)
      {
	MenuItem tempMenu = (MenuItem) evt.target;
	if (tempMenu.getLabel().equals("Quit"))
	  {
	    answers.dispose();
	    handled = super.action(evt, what);
	  }
	else if (evt.target == ansMenuItem)
	  {
	    answers.show();
	    answers.toFront();
	    tPanel.ShowAnswer();
	    tPanel.repaint();
	    handled = true;
	  }
	else
	  handled = super.action(evt, what);
      }
    else if (evt.target instanceof Button) {
      tPanel.HideAnswer();
      tPanel.repaint();
      handled = true;
    }
    else
      handled = super.action(evt, what);
    return handled;
  }

}
